@@ -31,49 +31,6 @@
 * instead of those above.
 */
 
-/*
- *  freely inspired from istorage :
- * ------------------------------------------------------------
- *      Generic OLE Zones furnished with a copy of the file header
- *
- * Compound Storage (32 bit version)
- * Storage implementation
- *
- * This file contains the compound file implementation
- * of the storage interface.
- *
- * Copyright 1999 Francis Beaudet
- * Copyright 1999 Sylvain St-Germain
- * Copyright 1999 Thuy Nguyen
- * Copyright 2005 Mike McCormack
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- */
-
-/*
- * NOTES
- *  The compound file implementation of IStorage used for create
- *  and manage substorages and streams within a storage object
- *  residing in a compound file object.
- *
- * MSDN
- *  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/stg/stg/istorage_compound_file_implementation.asp
- * ------------------------------------------------------------
- */
-
 #include <cstdlib>
 #include <cstring>
 #include <iostream>
@@ -83,6 +40,7 @@
 
 #include <librevenge/librevenge.h>
 
+#include "MWAWFontConverter.hxx"
 #include "MWAWPosition.hxx"
 #include "MWAWOLEParser.hxx"
 #include "MWAWPictMac.hxx"
@@ -228,16 +186,45 @@ protected:
 
 /** Internal: internal method to keep ole definition */
 struct OleDef {
-  OleDef() : m_id(-1), m_subId(-1), m_dir(""), m_name("") { }
+  OleDef() : m_id(-1), m_subId(-1), m_dir(""), m_base(""), m_name("") { }
   int m_id /**main id*/, m_subId /**subsversion id */ ;
-  std::string m_dir/**the directory*/, m_name/**the base name*/;
+  std::string m_dir/**the directory*/, m_base/**the base*/, m_name/**the complete name*/;
+};
+
+/** Internal: internal state of a MWAWOLEParser */
+struct State {
+  /** constructor */
+  State(MWAWFontConverterPtr fontConverter, int fId)
+    : m_fontConverter(fontConverter), m_fontId(fId), m_metaData(), m_unknownOLEs(),
+      m_objects(), m_objectsPosition(), m_objectsId(), m_objectsType(), m_compObjIdName()
+  {
+  }
+  //! the font converter
+  MWAWFontConverterPtr m_fontConverter;
+  //! the font id used to decode string
+  int m_fontId;
+  //! the meta data
+  librevenge::RVNGPropertyList m_metaData;
+  //! list of ole which can not be parsed
+  std::vector<std::string> m_unknownOLEs;
+
+  //! list of pictures read
+  std::vector<librevenge::RVNGBinaryData> m_objects;
+  //! list of picture size ( if known)
+  std::vector<MWAWPosition> m_objectsPosition;
+  //! list of pictures id
+  std::vector<int> m_objectsId;
+  //! list of picture type
+  std::vector<std::string> m_objectsType;
+
+  //! a smart ptr used to stored the list of compobj id->name
+  shared_ptr<MWAWOLEParserInternal::CompObj> m_compObjIdName;
 };
 }
 
 // constructor/destructor
-MWAWOLEParser::MWAWOLEParser(std::string mainName)
-  : m_avoidOLE(mainName), m_unknownOLEs(),
-    m_objects(), m_objectsPosition(), m_objectsId(), m_objectsType(), m_compObjIdName()
+MWAWOLEParser::MWAWOLEParser(std::string mainName, MWAWFontConverterPtr fontConverter, int fId)
+  : m_avoidOLE(mainName), m_state(new MWAWOLEParserInternal::State(fontConverter, fId))
 {
 }
 
@@ -245,13 +232,47 @@ MWAWOLEParser::~MWAWOLEParser()
 {
 }
 
+void MWAWOLEParser::updateMetaData(librevenge::RVNGPropertyList &metaData) const
+{
+  librevenge::RVNGPropertyList::Iter i(m_state->m_metaData);
+  for (i.rewind(); i.next();) {
+    if (!metaData[i.key()])
+      metaData.insert(i.key(),i()->clone());
+  }
+}
+
+std::vector<std::string> const &MWAWOLEParser::getNotParse() const
+{
+  return m_state->m_unknownOLEs;
+}
+
+std::vector<int> const &MWAWOLEParser::getObjectsId() const
+{
+  return m_state->m_objectsId;
+}
+
+std::vector<MWAWPosition> const &MWAWOLEParser::getObjectsPosition() const
+{
+  return m_state->m_objectsPosition;
+}
+
+std::vector<librevenge::RVNGBinaryData> const &MWAWOLEParser::getObjects() const
+{
+  return m_state->m_objects;
+}
+
+std::vector<std::string> const &MWAWOLEParser::getObjectsType() const
+{
+  return m_state->m_objectsType;
+}
+
 bool MWAWOLEParser::getObject(int id, librevenge::RVNGBinaryData &obj, MWAWPosition &pos, std::string &type)  const
 {
-  for (size_t i = 0; i < m_objectsId.size(); i++) {
-    if (m_objectsId[i] != id) continue;
-    obj = m_objects[i];
-    pos = m_objectsPosition[i];
-    type = m_objectsType[i];
+  for (size_t i = 0; i < m_state->m_objectsId.size(); i++) {
+    if (m_state->m_objectsId[i] != id) continue;
+    obj = m_state->m_objects[i];
+    pos = m_state->m_objectsPosition[i];
+    type = m_state->m_objectsType[i];
     return true;
   }
   obj.clear();
@@ -261,29 +282,29 @@ bool MWAWOLEParser::getObject(int id, librevenge::RVNGBinaryData &obj, MWAWPosit
 void MWAWOLEParser::setObject(int id, librevenge::RVNGBinaryData const &obj, MWAWPosition const &pos,
                               std::string const &type)
 {
-  for (size_t i = 0; i < m_objectsId.size(); i++) {
-    if (m_objectsId[i] != id) continue;
-    m_objects[i] = obj;
-    m_objectsPosition[i] = pos;
-    m_objectsType[i] = type;
+  for (size_t i = 0; i < m_state->m_objectsId.size(); i++) {
+    if (m_state->m_objectsId[i] != id) continue;
+    m_state->m_objects[i] = obj;
+    m_state->m_objectsPosition[i] = pos;
+    m_state->m_objectsType[i] = type;
     return;
   }
-  m_objects.push_back(obj);
-  m_objectsPosition.push_back(pos);
-  m_objectsId.push_back(id);
-  m_objectsType.push_back(type);
+  m_state->m_objects.push_back(obj);
+  m_state->m_objectsPosition.push_back(pos);
+  m_state->m_objectsId.push_back(id);
+  m_state->m_objectsType.push_back(type);
 }
 
 // parsing
 bool MWAWOLEParser::parse(MWAWInputStreamPtr file)
 {
-  if (!m_compObjIdName)
-    m_compObjIdName.reset(new MWAWOLEParserInternal::CompObj);
+  if (!m_state->m_compObjIdName)
+    m_state->m_compObjIdName.reset(new MWAWOLEParserInternal::CompObj);
 
-  m_unknownOLEs.resize(0);
-  m_objects.resize(0);
-  m_objectsId.resize(0);
-  m_objectsType.resize(0);
+  m_state->m_unknownOLEs.resize(0);
+  m_state->m_objects.resize(0);
+  m_state->m_objectsId.resize(0);
+  m_state->m_objectsType.resize(0);
 
   if (!file.get()) return false;
 
@@ -319,7 +340,8 @@ bool MWAWOLEParser::parse(MWAWInputStreamPtr file)
 #endif
     MWAWOLEParserInternal::OleDef data;
     data.m_name = name;
-    data.m_dir = base;
+    data.m_dir = dir;
+    data.m_base = base;
 
     // try to retrieve the identificator stored in the directory
     //  MatOST/MatadorObject1/ -> 1, -1
@@ -379,32 +401,34 @@ bool MWAWOLEParser::parse(MWAWInputStreamPtr file)
       bool ok = true;
       MWAWPosition pictPos;
 
-      if (strncmp("Ole", dOle.m_dir.c_str(), 3) == 0 ||
-          strncmp("CompObj", dOle.m_dir.c_str(), 7) == 0)
+      if (strncmp("Ole", dOle.m_base.c_str(), 3) == 0 ||
+          strncmp("CompObj", dOle.m_base.c_str(), 7) == 0)
         ole->setReadInverted(true);
 
       try {
-        if (readMM(ole, dOle.m_dir, asciiFile));
-        else if (readObjInfo(ole, dOle.m_dir, asciiFile));
-        else if (readOle(ole, dOle.m_dir, asciiFile));
-        else if (isOlePres(ole, dOle.m_dir) &&
+        librevenge::RVNGPropertyList pList;
+        if (readMM(ole, dOle.m_base, asciiFile));
+        else if (readSummaryInformation(ole, dOle.m_base, dOle.m_dir.empty() ? m_state->m_metaData : pList, asciiFile));
+        else if (readObjInfo(ole, dOle.m_base, asciiFile));
+        else if (readOle(ole, dOle.m_base, asciiFile));
+        else if (isOlePres(ole, dOle.m_base) &&
                  readOlePres(ole, data, pictPos, asciiFile)) {
           hasData = true;
           newConfidence = 2;
         }
-        else if (isOle10Native(ole, dOle.m_dir) &&
+        else if (isOle10Native(ole, dOle.m_base) &&
                  readOle10Native(ole, data, asciiFile)) {
           hasData = true;
           // small size can be a symptom that this is a link to a
           // basic msworks data file, so we reduce confidence
           newConfidence = data.size() > 1000 ? 4 : 2;
         }
-        else if (readCompObj(ole, dOle.m_dir, asciiFile));
-        else if (readContents(ole, dOle.m_dir, data, pictPos, asciiFile)) {
+        else if (readCompObj(ole, dOle.m_base, asciiFile));
+        else if (readContents(ole, dOle.m_base, data, pictPos, asciiFile)) {
           hasData = true;
           newConfidence = 3;
         }
-        else if (readCONTENTS(ole, dOle.m_dir, data, pictPos, asciiFile)) {
+        else if (readCONTENTS(ole, dOle.m_base, data, pictPos, asciiFile)) {
           hasData = true;
           newConfidence = 3;
         }
@@ -415,7 +439,7 @@ bool MWAWOLEParser::parse(MWAWInputStreamPtr file)
         ok = false;
       }
       if (!ok) {
-        m_unknownOLEs.push_back(dOle.m_name);
+        m_state->m_unknownOLEs.push_back(dOle.m_name);
         asciiFile.reset();
         continue;
       }
@@ -427,7 +451,7 @@ bool MWAWOLEParser::parse(MWAWInputStreamPtr file)
         if (dataInput) {
           dataInput->seek(0, librevenge::RVNG_SEEK_SET);
           MWAWBox2f box;
-          if (MWAWPictData::check(dataInput, (int)data.size(), box) != MWAWPict::MWAW_R_BAD) {
+          if (MWAWPictData::check(dataInput, static_cast<int>(data.size()), box) != MWAWPict::MWAW_R_BAD) {
             isPict = true;
             newConfidence = 100;
           }
@@ -460,18 +484,18 @@ bool MWAWOLEParser::parse(MWAWInputStreamPtr file)
     }
 
     if (pict.size()) {
-      m_objects.push_back(pict);
-      if (actualPos.naturalSize().x() <= 0. || actualPos.naturalSize().y() <= 0.) {
+      m_state->m_objects.push_back(pict);
+      if (actualPos.naturalSize().x() <= 0 || actualPos.naturalSize().y() <= 0) {
         MWAWVec2f size = potentialSize.naturalSize();
         if (size.x() > 0 && size.y() > 0)
           actualPos.setNaturalSize(actualPos.getInvUnitScale(potentialSize.unit())*size);
       }
-      m_objectsPosition.push_back(actualPos);
-      m_objectsId.push_back(id);
+      m_state->m_objectsPosition.push_back(actualPos);
+      m_state->m_objectsId.push_back(id);
       if (isPict)
-        m_objectsType.push_back("image/pict");
+        m_state->m_objectsType.push_back("image/pict");
       else
-        m_objectsType.push_back("object/ole");
+        m_state->m_objectsType.push_back("object/ole");
     }
   }
 
@@ -498,7 +522,7 @@ bool MWAWOLEParser::readOle(MWAWInputStreamPtr ip, std::string const &oleName,
 
   int val[20];
   for (int i= 0; i < 20; i++) {
-    val[i] = (int) ip->readLong(1);
+    val[i] = static_cast<int>(ip->readLong(1));
     if (val[i] < -10 || val[i] > 10) return false;
   }
 
@@ -548,7 +572,7 @@ bool MWAWOLEParser::readMM(MWAWInputStreamPtr input, std::string const &oleName,
   if (input->tell() != 14 || !input->isEnd()) return false;
 
   input->seek(0, librevenge::RVNG_SEEK_SET);
-  int entete = (int) input->readULong(2);
+  int entete = static_cast<int>(input->readULong(2));
   if (entete != 0x444e) {
     if (entete == 0x4e44) {
       MWAW_DEBUG_MSG(("MWAWOLEParser::readMM: ERROR: endian mode probably bad, potentially bad PC/Mac mode detection.\n"));
@@ -560,7 +584,7 @@ bool MWAWOLEParser::readMM(MWAWInputStreamPtr input, std::string const &oleName,
 
   int val[6];
   for (int i = 0; i < 6; i++)
-    val[i] = (int) input->readLong(2);
+    val[i] = static_cast<int>(input->readLong(2));
 
   switch (val[5]) {
   case 0:
@@ -609,7 +633,7 @@ bool MWAWOLEParser::readCompObj(MWAWInputStreamPtr ip, std::string const &oleNam
   ip->seek(0,librevenge::RVNG_SEEK_SET);
 
   for (int i = 0; i < 6; i++) {
-    int val = (int) ip->readLong(2);
+    int val = static_cast<int>(ip->readLong(2));
     f << val << ", ";
   }
 
@@ -626,11 +650,11 @@ bool MWAWOLEParser::readCompObj(MWAWInputStreamPtr ip, std::string const &oleNam
   f << "@@CompObj(CLSID):";
   if (clsData[1] == 0 && clsData[2] == 0xC0 && clsData[3] == 0x46000000L) {
     // normally, a referenced object
-    char const *clsName = m_compObjIdName->getCLSName(clsData[0]);
+    char const *clsName = m_state->m_compObjIdName->getCLSName(clsData[0]);
     if (clsName)
       f << "'" << clsName << "'";
     else {
-      MWAW_DEBUG_MSG(("MWAWOLEParser::readCompObj: unknown clsid=%ld\n", (long) clsData[0]));
+      MWAW_DEBUG_MSG(("MWAWOLEParser::readCompObj: unknown clsid=%ld\n", long(clsData[0])));
       f << "unknCLSID='" << std::hex << clsData[0] << "'";
     }
   }
@@ -662,7 +686,7 @@ bool MWAWOLEParser::readCompObj(MWAWInputStreamPtr ip, std::string const &oleNam
     }
     else {
       for (int i = 0; i < sz; i++)
-        st += (char) ip->readULong(1);
+        st += char(ip->readULong(1));
     }
 
     f.str("");
@@ -691,8 +715,13 @@ bool MWAWOLEParser::readCompObj(MWAWInputStreamPtr ip, std::string const &oleNam
   long nbElt = 4;
   if (ip->seek(actPos+16,librevenge::RVNG_SEEK_SET) != 0 ||
       ip->tell() != actPos+16) {
-    if ((ip->tell()-actPos)%4)
-      return false;
+    if ((ip->tell()-actPos)%4) {
+      f.str("");
+      f << "@@CompObj(Footer):###";
+      ascii.addPos(actPos);
+      ascii.addNote(f.str().c_str());
+      return true;
+    }
     nbElt = (ip->tell()-actPos)/4;
   }
 
@@ -709,6 +738,280 @@ bool MWAWOLEParser::readCompObj(MWAWInputStreamPtr ip, std::string const &oleNam
   return true;
 }
 
+//////////////////////////////////////////////////
+// summary and doc summary
+//////////////////////////////////////////////////
+bool MWAWOLEParser::readSummaryInformation(MWAWInputStreamPtr input, std::string const &oleName, librevenge::RVNGPropertyList &pList,
+    libmwaw::DebugFile &ascii) const
+{
+  if (oleName!="SummaryInformation" && oleName!="DocumentSummaryInformation") return false;
+  input->seek(0, librevenge::RVNG_SEEK_SET);
+  libmwaw::DebugStream f;
+  f << "Entries(SumInfo):";
+  bool isDoc=oleName=="DocumentSummaryInformation";
+  if (isDoc) f << "doc,";
+  int val=int(input->readULong(2));
+  if (val==0xfeff) {
+    input->setReadInverted(false);
+    val=0xfffe;
+  }
+  if (input->size()<48 || val!=0xfffe) {
+    MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: header seems bad\n"));
+    f << "###";
+    ascii.addPos(0);
+    ascii.addNote(f.str().c_str());
+    return true;
+  }
+  for (int i=0; i<11; ++i) { // f1=1, f2=0-2
+    val=int(input->readULong(2));
+    if (val) f << "f" << i << "=" << val << ",";
+  }
+  val=int(input->readULong(4));
+  if (val==0x1000000) {
+    val=1;
+    input->setReadInverted(!input->readInverted());
+  }
+  if (val!=1) {
+    MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: summary info is bad\n"));
+    f << "###sumInfo=" << val << ",";
+    ascii.addPos(0);
+    ascii.addNote(f.str().c_str());
+    return true;
+  }
+  for (int i=0; i<4; ++i) {
+    val=int(input->readULong(4));
+    static int const expected[]= {int(0xf29f85e0),0x10684ff9,0x891ab,int(0xd9b3272b)};
+    static int const docExpected[]= {int(0xd5cdd502),0x101b2e9c,0x89793,int(0xaef92c2b)};
+    if ((!isDoc && val==expected[i]) || (isDoc && val==docExpected[i])) continue;
+    f << "#fmid" << i << "=" << std::hex << val << std::dec << ",";
+    static bool first=true;
+    if (first) {
+      MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: fmid is bad\n"));
+      first=false;
+    }
+  }
+  int decal=int(input->readULong(4));
+  if (decal<0x30 || input->size()<decal) {
+    MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: decal is bad\n"));
+    f << "decal=" << val << ",";
+    ascii.addPos(0);
+    ascii.addNote(f.str().c_str());
+    return true;
+
+  }
+  ascii.addPos(0);
+  ascii.addNote(f.str().c_str());
+  if (decal!=0x30) {
+    ascii.addPos(0x30);
+    ascii.addNote("_");
+    input->seek(decal, librevenge::RVNG_SEEK_SET);
+  }
+
+  long pos=input->tell();
+  f.str("");
+  f << "SumInfo-A:";
+  long pSetSize=long(input->readULong(4));
+  int N=int(input->readULong(4));
+  f << "N=" << N << ",";
+  if (pSetSize<0 || input->size()-pos<pSetSize || (pSetSize-8)/8<N) {
+    MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: psetstruct is bad\n"));
+    f << "###";
+    ascii.addPos(pos);
+    ascii.addNote(f.str().c_str());
+    return true;
+  }
+  f << "[";
+  std::map<long,int> posToTypeMap;
+  for (int i=0; i<N; ++i) {
+    int type=int(input->readULong(4));
+    int depl=int(input->readULong(4));
+    if (depl==0) continue;
+    f << std::hex << depl << std::dec << ":" << type << ",";
+    if (depl<8+8*N || depl+4>pSetSize || posToTypeMap.find(pos+depl)!=posToTypeMap.end()) {
+      f << "###";
+      continue;
+    }
+    posToTypeMap[pos+depl]=type;
+  }
+  f << "],";
+  ascii.addPos(pos);
+  ascii.addNote(f.str().c_str());
+
+  for (std::map<long,int>::const_iterator it=posToTypeMap.begin(); it!=posToTypeMap.end(); ++it) {
+    pos=it->first;
+    long endPos=input->size();
+    std::map<long,int>::const_iterator nextIt=it;
+    if (++nextIt!=posToTypeMap.end()) endPos=nextIt->first;
+    input->seek(pos, librevenge::RVNG_SEEK_SET);
+    f.str("");
+    f << "SumInfo-B" << it->second << ":";
+    int type=int(input->readULong(4));
+    if (type==0x1e && !isDoc && ((it->second>=2 && it->second<=6) || it->second==8)) {
+      librevenge::RVNGString text;
+      if (readSummaryPropertyString(input, endPos, type, text, f) && !text.empty()) {
+        static char const *(attribNames[]) = {
+          "", "", "dc:title", "dc:subject", "meta:initial-creator",
+          "meta:keywords", "dc:description"/*comment*/, "", "dc:creator"
+        };
+        pList.insert(attribNames[it->second], text);
+      }
+    }
+    else if (!readSummaryProperty(input, endPos, type, ascii, f)) {
+      MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: find unknown type\n"));
+      f << "##type=" << std::hex << type << std::dec << ",";
+    }
+    if (input->tell()!=endPos && input->tell()!=pos)
+      ascii.addDelimiter(input->tell(),'|');
+    ascii.addPos(pos);
+    ascii.addNote(f.str().c_str());
+  }
+  return true;
+}
+
+bool MWAWOLEParser::readSummaryPropertyString(MWAWInputStreamPtr input, long endPos, int type,
+    librevenge::RVNGString &string, libmwaw::DebugStream &f) const
+{
+  if (!input) return false;
+  long pos=input->tell();
+  string.clear();
+  long sSz=long(input->readULong(4));
+  if (sSz<0 || (endPos-pos-4)<sSz || pos+4+sSz>endPos) {
+    MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryPropertyString: string size is bad\n"));
+    f << "##stringSz=" << sSz << ",";
+    return false;
+  }
+  std::string text("");
+  for (long c=0; c < sSz; ++c) {
+    char ch=char(input->readULong(1));
+    if (ch) {
+      text+=ch;
+      if (m_state->m_fontConverter) {
+        int unicode=m_state->m_fontConverter->unicode(m_state->m_fontId, static_cast<unsigned char>(ch));
+        if (unicode!=-1)
+          libmwaw::appendUnicode(uint32_t(unicode), string);
+      }
+    }
+    else if (c+1!=sSz)
+      text+="##";
+  }
+  f << text;
+  if (type==0x1f && (sSz%4))
+    input->seek(sSz%4, librevenge::RVNG_SEEK_CUR);
+  return true;
+}
+
+bool MWAWOLEParser::readSummaryProperty(MWAWInputStreamPtr input, long endPos, int type,
+                                        libmwaw::DebugFile &ascii, libmwaw::DebugStream &f) const
+{
+  if (!input) return false;
+  long pos=input->tell();
+  // see propread.cxx
+  if (type&0x1000) {
+    int N=int(input->readULong(4));
+    f << "N=" << N << ",";
+    f << "[";
+    for (int n=0; n<N; ++n) {
+      pos=input->tell();
+      f << "[";
+      if (!readSummaryProperty(input, endPos, type&0xFFF, ascii, f)) {
+        input->seek(pos, librevenge::RVNG_SEEK_SET);
+        return false;
+      }
+      f << "],";
+    }
+    f << "],";
+    return true;
+  }
+  switch (type) {
+  case 0x10: // int1
+  case 0x11: // uint1
+    if (pos+1>endPos)
+      return false;
+    f << "val=" << char(input->readULong(1));
+    break;
+  case 2: // int
+  case 0xb: // bool
+  case 0x12: // uint
+    if (pos+2>endPos)
+      return false;
+    if (type==2)
+      f << "val=" << int(input->readLong(2)) << ",";
+    else if (type==0x12)
+      f << "val=" << int(input->readULong(2)) << ",";
+    else if (input->readULong(2))
+      f << "true,";
+    break;
+  case 3: // int
+  case 4: // float
+  case 9: // uint
+    if (pos+4>endPos)
+      return false;
+    if (type==3)
+      f << "val=" << int(input->readLong(4)) << ",";
+    else if (type==9)
+      f << "val=" << int(input->readULong(4)) << ",";
+    else
+      f << "val[fl4]=" << std::hex << input->readULong(4) << std::dec << ",";
+    break;
+  case 5: // double
+  case 6:
+  case 7:
+  case 20:
+  case 21:
+  case 0x40:
+    if (pos+8>endPos)
+      return false;
+    ascii.addDelimiter(input->tell(),'|');
+    if (type==5)
+      f << "double,";
+    else if (type==6)
+      f << "cy,";
+    else if (type==7)
+      f << "date,";
+    else if (type==20)
+      f << "long,";
+    else if (type==21)
+      f << "ulong,";
+    else
+      f << "fileTime,"; // readme 8 byte
+    input->seek(pos+8, librevenge::RVNG_SEEK_SET);
+    break;
+  case 0xc: // variant
+    if (pos+4>endPos)
+      return false;
+    type=int(input->readULong(4));
+    return readSummaryProperty(input, endPos, type, ascii, f);
+  // case 20: int64
+  // case 21: uint64
+  case 8:
+  case 0x1e:
+  case 0x1f: {
+    librevenge::RVNGString string;
+    if (!readSummaryPropertyString(input, endPos, type, string, f))
+      return false;
+    break;
+  }
+  case 0x41:
+  case 0x46:
+  case 0x47: {
+    if (pos+4>endPos)
+      return false;
+    f << (type==0x41 ? "blob" : type==0x46 ? "blob[object]" : "clipboard") << ",";
+    long dSz=long(input->readULong(4));
+    if (dSz<0 || pos+4+dSz>endPos)
+      return false;
+    if (dSz) {
+      ascii.skipZone(pos+4, pos+4+dSz-1);
+      input->seek(dSz, librevenge::RVNG_SEEK_CUR);
+    }
+    break;
+  }
+  /* todo type==0x47, vtcf clipboard */
+  default:
+    return false;
+  }
+  return true;
+}
 //////////////////////////////////////////////////
 //
 // OlePres001 seems to contained standart picture file and size
@@ -799,12 +1102,12 @@ bool MWAWOLEParser::readOlePres(MWAWInputStreamPtr ip, librevenge::RVNGBinaryDat
       std::string str;
       bool find = false;
       while (ip->tell() < endHPos) {
-        unsigned char c = (unsigned char)ip->readULong(1);
+        unsigned char c = static_cast<unsigned char>(ip->readULong(1));
         if (c == 0) {
           find = true;
           break;
         }
-        str += (char) c;
+        str += char(c);
       }
       if (!find) {
         ok = false;
@@ -832,8 +1135,8 @@ bool MWAWOLEParser::readOlePres(MWAWInputStreamPtr ip, librevenge::RVNGBinaryDat
     f << val << ", ";
   }
   // dim in TWIP ?
-  long extendX = (long) ip->readULong(4);
-  long extendY = (long)ip->readULong(4);
+  long extendX = long(ip->readULong(4));
+  long extendY = long(ip->readULong(4));
   if (extendX > 0 && extendY > 0) pos.setNaturalSize(MWAWVec2f(float(extendX)/20.f, float(extendY)/20.f));
   long fSize = ip->readLong(4);
   f << "extendX="<< extendX << ", extendY=" << extendY << ", fSize=" << fSize;
@@ -939,7 +1242,7 @@ bool MWAWOLEParser::readContents(MWAWInputStreamPtr input,
   f << "bdbox0=(" << dim[0] << "," << dim[1]<<"),";
   for (int i = 0; i < 3; i++) {
     /* 0,{10|21|75|101|116}x2 */
-    long val = (long) input->readULong(4);
+    long val = long(input->readULong(4));
     if (val < 1000)
       f << val << ",";
     else
@@ -970,7 +1273,7 @@ bool MWAWOLEParser::readContents(MWAWInputStreamPtr input,
   }
 
   long actPos = input->tell();
-  long size = (long) input->readULong(4);
+  long size = long(input->readULong(4));
   if (size <= 0) ok = false;
   if (ok) {
     input->seek(actPos+size+4, librevenge::RVNG_SEEK_SET);
@@ -1031,7 +1334,7 @@ bool MWAWOLEParser::readCONTENTS(MWAWInputStreamPtr input,
   input->seek(0, librevenge::RVNG_SEEK_SET);
   f << "@@CONTENTS:";
 
-  long hSize = (long)input->readULong(4);
+  long hSize = long(input->readULong(4));
   if (input->isEnd()) return false;
   f << "hSize=" << std::hex << hSize << std::dec;
 
@@ -1044,9 +1347,9 @@ bool MWAWOLEParser::readCONTENTS(MWAWInputStreamPtr input,
 
   // minimal checking of the "copied" header
   input->seek(4, librevenge::RVNG_SEEK_SET);
-  long type = (long) input->readULong(4);
+  long type = long(input->readULong(4));
   if (type < 0 || type > 4) return false;
-  long newSize = (long) input->readULong(4);
+  long newSize = long(input->readULong(4));
 
   f << ", type=" << type;
   if (newSize < 8) return false;
@@ -1068,15 +1371,15 @@ bool MWAWOLEParser::readCONTENTS(MWAWInputStreamPtr input,
     f << "=(" << dim[0] << "x" << dim[1] << "<->" << dim[2] << "x" << dim[3] << ")";
   }
   char dataType[5];
-  for (int i = 0; i < 4; i++) dataType[i] = (char) input->readULong(1);
+  for (int i = 0; i < 4; i++) dataType[i] = char(input->readULong(1));
   dataType[4] = '\0';
   f << ",typ=\""<<dataType<<"\""; // always " EMF" ?
 
   for (int i = 0; i < 2; i++) { // always id0=0, id1=1 ?
-    int val = (int) input->readULong(2);
+    int val = static_cast<int>(input->readULong(2));
     if (val) f << ",id"<< i << "=" << val;
   }
-  long dataLength = (long) input->readULong(4);
+  long dataLength = long(input->readULong(4));
   f << ",length=" << dataLength+hSize;
 
   ascii.addPos(0);
@@ -1090,14 +1393,14 @@ bool MWAWOLEParser::readCONTENTS(MWAWInputStreamPtr input,
     // or f0=a,f1=1,f2=2,f3=6c,f5=480,f6=360,f7=140,f8=f0
     // or f0=61,f1=1,f2=2,f3=58,f5=280,f6=1e0,f7=a9,f8=7f
     // f3=some header sub size ? f5/f6 and f7/f8 two other bdbox ?
-    long val = (long) input->readULong(4);
+    long val = long(input->readULong(4));
     if (val) f << std::hex << ",f" << i << "=" << val;
   }
   for (int i = 0; 2*i+100 < hSize; i++) {
     // g0=e3e3,g1=6,g2=4e6e,g3=4
     // g0=e200,g1=4,g2=a980,g3=3,g4=4c,g5=50
     // ---
-    long val = (long) input->readULong(2);
+    long val = long(input->readULong(2));
     if (val) f << std::hex << ",g" << i << "=" << val;
   }
   ascii.addNote(f.str().c_str());
